@@ -5,15 +5,20 @@ There's some Python 2.x <-> 3.x compatibility code here.
 Pyro - Python Remote Objects.  Copyright by Irmen de Jong (irmen@razorvine.net).
 """
 
-from __future__ import with_statement
 import sys
-import threading
 import pickle
-import Pyro4
+import threading
+from Pyro4 import errors, core, expose, behavior, current_context
+from Pyro4.configuration import config
 
-__all__ = ["tobytes", "tostring", "unicode", "unichr", "basestring", "StringIO", "next",
-           "AtomicCounter", "NonserializableError", "MyThing", "MyThingExposed",
-           "MyThingExposedSub", "MyThingSub", "unittest"]
+
+__all__ = ["tobytes", "tostring", "unicode", "unichr", "basestring", "StringIO",
+           "NonserializableError", "MyThingPartlyExposed", "MyThingFullExposed",
+           "MyThingExposedSub", "MyThingPartlyExposedSub", "ConnectionMock",
+           "AtomicCounter", "ResourceService", "Resource"]
+
+
+config.reset(False)   # reset the config to default
 
 if sys.version_info < (3, 0):
     # noinspection PyUnresolvedReferences
@@ -41,42 +46,13 @@ else:
     unichr = chr
     basestring = str
 
-if sys.version_info < (2, 6):
-    def next(iterable):
-        return iterable.next()
-else:
-    next = next
-
-if ((2, 7) <= sys.version_info < (3, 0)) or (sys.version_info >= (3, 1)):
-    import unittest
-else:
-    # noinspection PyUnresolvedReferences
-    import unittest2 as unittest
-
-
-class AtomicCounter(object):
-    def __init__(self):
-        self.lock = threading.Lock()
-        self.count = 0
-
-    def reset(self):
-        self.count = 0
-
-    def incr(self):
-        with self.lock:
-            self.count += 1
-
-    def value(self):
-        with self.lock:
-            return self.count
-
 
 class NonserializableError(Exception):
     def __reduce__(self):
         raise pickle.PicklingError("to make this error non-serializable")
 
 
-class MyThing(object):
+class MyThingPartlyExposed(object):
     c_attr = "hi"
     propvalue = 42
     _private_attr1 = "hi"
@@ -87,7 +63,7 @@ class MyThing(object):
         self.name = name
 
     def __eq__(self, other):
-        if type(other) is MyThing:
+        if type(other) is MyThingPartlyExposed:
             return self.name == other.name
         return False
 
@@ -102,7 +78,7 @@ class MyThing(object):
     def classmethod(cls, arg):
         pass
 
-    def __private__(self):
+    def __dunder__(self):
         pass
 
     def __private(self):
@@ -111,17 +87,17 @@ class MyThing(object):
     def _private(self):
         pass
 
-    @Pyro4.expose
+    @core.expose
     @property
     def prop1(self):
         return self.propvalue
 
-    @Pyro4.expose
+    @core.expose
     @prop1.setter
     def prop1(self, value):
         self.propvalue = value
 
-    @Pyro4.expose
+    @core.expose
     @property
     def readonly_prop1(self):
         return self.propvalue
@@ -134,78 +110,186 @@ class MyThing(object):
     def prop2(self, value):
         self.propvalue = value
 
-    @Pyro4.oneway
+    @core.oneway
+    @core.expose
     def oneway(self, arg):
         pass
 
-    @Pyro4.expose
+    @core.expose
     def exposed(self):
         pass
 
     __hash__ = object.__hash__
 
 
-@Pyro4.expose
-class MyThingExposed(object):
-    blurp = 99   # won't be exposed, because it is a class attribute and not a property
-    _name = ""
+@core.expose
+class MyThingFullExposed(object):
+    """this is the same as MyThingPartlyExposed but the whole class should be exposed"""
+    c_attr = "hi"
+    propvalue = 42
+    _private_attr1 = "hi"
+    __private_attr2 = "hi"
+    name = ""
 
     def __init__(self, name="dummy"):
-        self._name = name
+        self.name = name    # note: not affected by @expose, only real properties are
 
     def __eq__(self, other):
-        if type(other) is MyThingExposed:
-            return self._name == other._name
+        if type(other) is MyThingFullExposed:
+            return self.name == other.name
         return False
 
-    def foo(self, arg):
-        return arg
-
-    @classmethod
-    def classmethod(cls, arg):
-        return arg
+    def method(self, arg, default=99, **kwargs):
+        pass
 
     @staticmethod
     def staticmethod(arg):
-        return arg
-
-    @property
-    def name(self):
-        return self._name
-
-    @name.setter
-    def name(self, value):
-        self._name = value
-
-    @property
-    def readonly_name(self):
-        return self._name
-
-    @Pyro4.oneway
-    def remotemethod(self, arg):
-        return arg
-
-    def _p(self):
         pass
 
-    def __private__(self):
+    @classmethod
+    def classmethod(cls, arg):
+        pass
+
+    def __dunder__(self):
+        pass
+
+    def __private(self):
+        pass
+
+    def _private(self):
+        pass
+
+    @property
+    def prop1(self):
+        return self.propvalue
+
+    @prop1.setter
+    def prop1(self, value):
+        self.propvalue = value
+
+    @property
+    def readonly_prop1(self):
+        return self.propvalue
+
+    @property
+    def prop2(self):
+        return self.propvalue
+
+    @prop2.setter
+    def prop2(self, value):
+        self.propvalue = value
+
+    @core.oneway
+    def oneway(self, arg):
+        pass
+
+    def exposed(self):
         pass
 
     __hash__ = object.__hash__
 
 
-class MyThingExposedSub(MyThingExposed):
+@core.expose
+class MyThingExposedSub(MyThingFullExposed):
     def sub_exposed(self):
         pass
 
     def sub_unexposed(self):
         pass
 
+    @core.oneway
+    def oneway2(self):
+        pass
 
-class MyThingSub(MyThing):
-    @Pyro4.expose
+
+class MyThingPartlyExposedSub(MyThingPartlyExposed):
+    @core.expose
     def sub_exposed(self):
         pass
 
     def sub_unexposed(self):
         pass
+
+    @core.oneway
+    def oneway2(self):
+        pass
+
+
+class ConnectionMock(object):
+    def __init__(self, initial_msg=None):
+        self.keep_open = False
+        if not initial_msg:
+            self.received = b""
+        elif isinstance(initial_msg, (str, bytes)):
+            self.received = initial_msg
+        else:
+            self.received = initial_msg.to_bytes()   # it's probably a Message object
+
+    def send(self, data):
+        self.received += data
+
+    def recv(self, datasize):
+        chunk = self.received[:datasize]
+        self.received = self.received[datasize:]
+        if len(chunk) < datasize:
+            raise errors.ConnectionClosedError("receiving: not enough data")
+        return chunk
+
+
+class AtomicCounter(object):
+    def __init__(self, value=0):
+        self.__initial = value
+        self.__value = value
+        self.__lock = threading.Lock()
+
+    def reset(self):
+        self.__value = self.__initial
+
+    def incr(self, amount=1):
+        with self.__lock:
+            self.__value += amount
+            return self.__value
+
+    def decr(self, amount=1):
+        with self.__lock:
+            self.__value -= amount
+            return self.__value
+
+    @property
+    def value(self):
+        return self.__value
+
+
+class Resource(object):
+    # a fictional resource that gets allocated and must be freed again later.
+    def __init__(self, name, collection):
+        self.name = name
+        self.collection = collection
+        self.close_called = False
+
+    def close(self):
+        # Pyro will call this on a tracked resource once the client's connection gets closed!
+        self.collection.discard(self)
+        self.close_called = True
+
+
+@expose
+@behavior(instance_mode="single")
+class ResourceService(object):
+    def __init__(self):
+        self.resources = set()      # the allocated resources
+
+    def allocate(self, name):
+        resource = Resource(name, self.resources)
+        self.resources.add(resource)
+        current_context.track_resource(resource)
+
+    def free(self, name):
+        resources = {r for r in self.resources if r.name == name}
+        self.resources -= resources
+        for r in resources:
+            r.close()
+            current_context.untrack_resource(r)
+
+    def list(self):
+        return [r.name for r in self.resources]
